searchNodes=[{"doc":"Luminous Luminous is a framework for creating dashboards within Phoenix Live View . Dashboards are defined by the client application (framework consumer) using elixir code and consist of Panels ( Luminous.Panel ) which are responsible for visualizing the results of multiple client-side queries ( Luminous.Query ). Three different types of Panels are currently offered out of the box by Luminous: Luminous.Panel.Chart for visualizing 2-d data (including time series) using the chartjs library (embedded in a JS hook). Currently, only :line and :bar are supported. Luminous.Panel.Stat for displaying single or multiple numerical or other values (e.g. strings) Luminous.Panel.Table for displaying tabular data A client application can implement its own custom panels by implementing the Luminous.Panel behaviour. Dashboards are parameterized by: a date range (using the flatpickr library) user-defined variables ( Luminous.Variable ) in the form of dropdown menus All panels are refreshed whenever at least one of these paramaters (date range, variables) change. The parameter values are available to client-side queries. Features Date range selection and automatic asynchronous (i.e. non-blocking for the UI) refresh of all dashboard panel queries User-facing variable dropdowns (with single- or multi- selection) whose selected values are available to panel queries Client-side zoom in charts with automatic update of the entire dashboard with the new date range Panel data downloads depending on the panel type (CSV, PNG) Stat panels (show single or multiple stats) Table panels using tabulator Summary statistics in charts Installation The package can be installed from hex.pm as follows: def deps do [ { :luminous , &quot;~&gt; 2.4.0&quot; } ] end In order to be able to use the provided components, the library's javascript and CSS files must be imported to your project: In assets/js/app.js : import { ChartJSHook, TableHook, TimeRangeHook, MultiSelectVariableHook } from &quot;luminous&quot; let Hooks = { TimeRangeHook: new TimeRangeHook(), ChartJSHook: new ChartJSHook(), TableHook: new TableHook(), MultiSelectVariableHook: new MultiSelectVariableHook() } ... let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, { ... hooks: Hooks }) ... Finally, in assets/css/app.css : @import &quot;../../deps/luminous/dist/luminous.css&quot;; Usage Live View The dashboard live view is defined client-side like so: defmodule ClientApp.DashboardLive do alias ClientApp.Router.Helpers , as : Routes use Luminous.Live , title : &quot;My Title&quot; , path : &amp; Routes . dashboard_path / 3 , action : :index , time_zone : &quot;Europe/Paris&quot; , panels : [ ... ] , variables : [ ... ] # the dashboard can be rendered by leveraging the corresponding functionality # from `Luminous.Components` def render ( assigns ) do ~H&quot;&quot;&quot; &lt;Luminous.Components.dashboard dashboard={@dashboard} /&gt; &quot;&quot;&quot; end end The client-side dashboard can also (optionally) implement the Luminous.TimeRange behaviour in order to override the dashboard's default time range value which is &quot;today&quot;. Panels and Queries Client-side queries must be included in a module that implements the Luminous.Query behaviour: defmodule ClientApp.DashboardLive do defmodule Queries do @behaviour Luminous.Query @impl true def query ( :my_query , _time_range , _variables ) do [ [ { :time , ~U[2022-08-19T10:00:00Z] } , { &quot;foo&quot; , 10 } , { &quot;bar&quot; , 100 } ] , [ { :time , ~U[2022-08-19T11:00:00Z] } , { &quot;foo&quot; , 11 } , { &quot;bar&quot; , 101 } ] ] end end use Luminous.Live , ... panels : [ Panel . define! ( type : Luminous.Panel.Chart , id : :simple_time_series , title : &quot;Simple Time Series&quot; , queries : [ Luminous.Query . define ( :my_query , Queries ) ] , description : &quot;&quot;&quot; This will be rendered as a tooltip when hovering over the panel&#39;s title &quot;&quot;&quot; ) , ] , ... end A panel may include multiple queries. When a panel is automatically refreshed, the execution flow is as follows: for each query: execute the user query callback execute the panel's transform/2 callback with the query result output aggregate the transformed query results update the dashboard state variable with the panel's data (possible server-side re-rendering) send a JS event to the browser (for panel hooks) The above flow needs to be understood when implementing custom panels. If the client application uses the panels provided by luminous, then the panel refresh flow is handled automatically and only use Luminous.Live with the appropriate options is necessary. Variables Variables represent user-facing elements in the form of dropdowns in which the user can select single (variable type: :single ) or multiple (variable type: :multi ) values. Variable selections trigger the refresh of all panels in the dashboard. The state of all variables is available within the query callback that is implemented by the client application. Just like queries, variables must be included in a module that implements the Luminous.Variable behaviour: defmodule ClientApp.DashboardLive do defmodule Variables do @behaviour Luminous.Variable @impl true def variable ( :simple_var , _assigns ) , do : [ &quot;hour&quot; , &quot;day&quot; , &quot;week&quot; ] def variable ( :descriptive_var , _assigns ) do [ %{ label : &quot;Visible Value 1&quot; , value : &quot;val1&quot; } , %{ label : &quot;Visible Value 2&quot; , value : &quot;val2&quot; } , ] end end use Luminous.Live , ... variables : [ Luminous.Variable . define! ( id : :simple_var , label : &quot;Select one value&quot; , module : Variables ) , Luminous.Variable . define! ( id : :descriptive_var , label : &quot;Select one value&quot; , module : Variables ) , ] , ... end The variable callback will receive the live view socket assigns as the second argument, however it is important to note that the variable/2 callback is executed once when the dashboard is loaded for populating the dropdown values. Demo Luminous provides a demo dashboard that showcases some of Luminous' capabilities. The demo dashboard can be inspected live using the project's development server (run mix run in the project and then visit this page ).","ref":"Luminous.html","title":"Luminous","type":"module"},{"doc":"Attributes map variable values (user-defined) to attribute keyword lists. They are created by parsing and validating a NimbleOptions schema (see parse/2 ).","ref":"Luminous.Attributes.html","title":"Luminous.Attributes","type":"module"},{"doc":"Parse the supplied keyword list using the specified schema (performs validations as well)","ref":"Luminous.Attributes.html#parse/2","title":"Luminous.Attributes.parse/2","type":"function"},{"doc":"Parse the supplied keyword list using the specified schema (performs validations as well). Raises on error.","ref":"Luminous.Attributes.html#parse!/2","title":"Luminous.Attributes.parse!/2","type":"function"},{"doc":"","ref":"Luminous.Attributes.html#t:t/0","title":"Luminous.Attributes.t/0","type":"type"},{"doc":"Attribute schemas that are common across all instances of their type.","ref":"Luminous.Attributes.Schema.html","title":"Luminous.Attributes.Schema","type":"module"},{"doc":"Schema for the attributes that apply to all data attributes regardless of the Panel in which they are defined","ref":"Luminous.Attributes.Schema.html#data/0","title":"Luminous.Attributes.Schema.data/0","type":"function"},{"doc":"Schema for the attributes that apply to all panels regardless of their type","ref":"Luminous.Attributes.Schema.html#panel/0","title":"Luminous.Attributes.Schema.panel/0","type":"function"},{"doc":"","ref":"Luminous.Attributes.Schema.html#t:t/0","title":"Luminous.Attributes.Schema.t/0","type":"type"},{"doc":"Phoenix function components for visualizing a dashboard and its constituent components.","ref":"Luminous.Components.html","title":"Luminous.Components","type":"module"},{"doc":"The dashboard component is responsible for rendering all the necessary elements: title variables time range selector panels Additionally, it registers callbacks for reacting to panel loading states. Attributes dashboard ( :map ) (required)","ref":"Luminous.Components.html#dashboard/1","title":"Luminous.Components.dashboard/1","type":"function"},{"doc":"This component is responsible for rendering a Panel by rendering all the common panel elements and then delegating the rendering to the concrete Panel Attributes panel ( :map ) (required) dashboard ( :map ) (required)","ref":"Luminous.Components.html#panel/1","title":"Luminous.Components.panel/1","type":"function"},{"doc":"This component is responsible for setting up various dashboard prerequisites","ref":"Luminous.Components.html#setup/1","title":"Luminous.Components.setup/1","type":"function"},{"doc":"This component is responsible for rendering the Luminous.TimeRange component. It consists of a date range picker and a presets dropdown. Attributes dashboard ( :map ) (required)","ref":"Luminous.Components.html#time_range/1","title":"Luminous.Components.time_range/1","type":"function"},{"doc":"This component is responsible for rendering the dropdown of the assigned Variable . Attributes variable ( :map ) (required)","ref":"Luminous.Components.html#variable/1","title":"Luminous.Components.variable/1","type":"function"},{"doc":"A dashboard is the highest-level luminous component and contains all the necessary dashboard attributes such as the panels, variables and the time range selector. It also stores the state of the panels (query results). The dashboard is initialized in Luminous.Live and populated at runtime using populate/2 .","ref":"Luminous.Dashboard.html","title":"Luminous.Dashboard","type":"module"},{"doc":"Parse the supplied parameters and return the dashboard map structure. The following options are supported: :title ( String.t/0 ) - Required. :path (function of arity 3) - Required. :action ( atom/0 ) - Required. :panels (list of map/0 ) - The default value is [] . :variables (list of map/0 ) - The default value is [] . :time_range_selector (struct of type Luminous.TimeRangeSelector) - The default value is %Luminous.TimeRangeSelector{id: nil, current_time_range: nil} . :time_zone ( String.t/0 ) - The default value is &quot;Europe/Athens&quot; .","ref":"Luminous.Dashboard.html#define!/1","title":"Luminous.Dashboard.define!/1","type":"function"},{"doc":"return the panel data for the specified panel","ref":"Luminous.Dashboard.html#get_data/2","title":"Luminous.Dashboard.get_data/2","type":"function"},{"doc":"Returns the LV path for the specific dashboard based on its configuration.","ref":"Luminous.Dashboard.html#path/3","title":"Luminous.Dashboard.path/3","type":"function"},{"doc":"Populate the dashboard's dynamic properties (e.g. variable values, time range etc.) at runtime.","ref":"Luminous.Dashboard.html#populate/2","title":"Luminous.Dashboard.populate/2","type":"function"},{"doc":"Update the dashboard's current time range","ref":"Luminous.Dashboard.html#update_current_time_range/2","title":"Luminous.Dashboard.update_current_time_range/2","type":"function"},{"doc":"Update the dashboard's panel data","ref":"Luminous.Dashboard.html#update_data/3","title":"Luminous.Dashboard.update_data/3","type":"function"},{"doc":"Update the dashboard's variables","ref":"Luminous.Dashboard.html#update_variables/2","title":"Luminous.Dashboard.update_variables/2","type":"function"},{"doc":"","ref":"Luminous.Dashboard.html#t:t/0","title":"Luminous.Dashboard.t/0","type":"type"},{"doc":"This module defines a macro that contains the functionality of a dashboard LiveView. It needs to be used ( use Luminous.Live ) inside a client application module with the appropriate options (as specified in Luminous.Dashboard.define!/1 ). More details and examples in the project README.","ref":"Luminous.Live.html","title":"Luminous.Live","type":"module"},{"doc":"A panel represents a single visual element (chart) in a dashboard that can contain many queries. A panel is &quot;refreshed&quot; when the live view first loads, as well as when a variable or the time range are updated. The panel's data (as returned by the queries) are stored in Luminous.Dashboard . The module defines a behaviour that must be implemented by concrete panels either inside Luminous (e.g. Luminous.Panel.Chart , Luminous.Panel.Stat etc.) or on the client side. When a Panel is refreshed ( refresh/3 ), the execution flow is as follows: for each query: execute the query ( Luminous.Query.execute() ) transform/2 the query result reduce/3 (aggregate) the transformed query results","ref":"Luminous.Panel.html","title":"Luminous.Panel","type":"behaviour"},{"doc":"A list of the available panel actions that will be transmitted as events using JS.dispatch using the format &quot;panel:${panel_id}:${event}&quot; The label will be shown in the dropdown. This is an optional callback -- if undefined (or if it returns []), then the dropdown is not rendered","ref":"Luminous.Panel.html#c:actions/0","title":"Luminous.Panel.actions/0","type":"callback"},{"doc":"Define the panel type's supported data attributes These will be used to parse, validate and populate the client's input","ref":"Luminous.Panel.html#c:data_attributes/0","title":"Luminous.Panel.data_attributes/0","type":"callback"},{"doc":"Initialize a panel. Verifies all supplied options both generic and the concrete panel's attributes. Will raise if the validation fails. The supported generic attributes are: :type ( atom/0 ) - Required. :id ( atom/0 ) - Required. :title ( String.t/0 ) - The default value is &quot;&quot; . :queries (list of struct of type Luminous.Query) :description - The default value is nil . :attributes ( keyword/0 ) - The default value is [] . :data_attributes - The default value is %{} .","ref":"Luminous.Panel.html#define!/1","title":"Luminous.Panel.define!/1","type":"function"},{"doc":"Define custom attributes specific to the concrete panel type These will be used to parse, validate and populate the client's input","ref":"Luminous.Panel.html#c:panel_attributes/0","title":"Luminous.Panel.panel_attributes/0","type":"callback"},{"doc":"aggregate all transformed results to a single map that will be sent for visualization","ref":"Luminous.Panel.html#c:reduce/3","title":"Luminous.Panel.reduce/3","type":"callback"},{"doc":"Refresh all panel queries.","ref":"Luminous.Panel.html#refresh/3","title":"Luminous.Panel.refresh/3","type":"function"},{"doc":"The phoenix function component that renders the panel. The panel's title, description tooltip, contextual menu etc. are rendered elsewhere. See Luminous.Components.panel/1 for a description of the available assigns.","ref":"Luminous.Panel.html#c:render/1","title":"Luminous.Panel.render/1","type":"callback"},{"doc":"transform a query result to view data acc. to the panel type","ref":"Luminous.Panel.html#c:transform/2","title":"Luminous.Panel.transform/2","type":"callback"},{"doc":"","ref":"Luminous.Panel.html#t:t/0","title":"Luminous.Panel.t/0","type":"type"},{"doc":"","ref":"Luminous.Panel.Chart.html","title":"Luminous.Panel.Chart","type":"module"},{"doc":"Attributes stats ( :map ) (required)","ref":"Luminous.Panel.Chart.html#panel_statistics/1","title":"Luminous.Panel.Chart.panel_statistics/1","type":"function"},{"doc":"","ref":"Luminous.Panel.Chart.html#statistics/2","title":"Luminous.Panel.Chart.statistics/2","type":"function"},{"doc":"","ref":"Luminous.Panel.Stat.html","title":"Luminous.Panel.Stat","type":"module"},{"doc":"","ref":"Luminous.Panel.Stat.html#transform/1","title":"Luminous.Panel.Stat.transform/1","type":"function"},{"doc":"","ref":"Luminous.Panel.Table.html","title":"Luminous.Panel.Table","type":"module"},{"doc":"A query is embedded in a panel and contains a function which will be executed upon panel refresh to fetch the query's data.","ref":"Luminous.Query.html","title":"Luminous.Query","type":"behaviour"},{"doc":"Initialize a query at compile time. The module must implement the Luminous.Query behaviour.","ref":"Luminous.Query.html#define/2","title":"Luminous.Query.define/2","type":"function"},{"doc":"Execute the query and return the data as multiple TimeSeries structs.","ref":"Luminous.Query.html#execute/3","title":"Luminous.Query.execute/3","type":"function"},{"doc":"A module must implement this behaviour to be passed as an argument to define/2 . A query must return a list of 2-tuples: the 2-tuple's first element is the time series' label the 2-tuple's second element is the label's value the list must contain a 2-tuple with the label :time and a DateTime value.","ref":"Luminous.Query.html#c:query/3","title":"Luminous.Query.query/3","type":"callback"},{"doc":"","ref":"Luminous.Query.html#t:result/0","title":"Luminous.Query.result/0","type":"type"},{"doc":"","ref":"Luminous.Query.html#t:t/0","title":"Luminous.Query.t/0","type":"type"},{"doc":"This module defines a struct with two fields ( :from and :to ) to represent a time range. Additionally, various helper functions are defined that operate on time ranges. It also specifies a behaviour that can be (optionally) implemented by client-side dashboards in order to override the dashboard's default time range (which is &quot;today&quot;).","ref":"Luminous.TimeRange.html","title":"Luminous.TimeRange","type":"behaviour"},{"doc":"","ref":"Luminous.TimeRange.html#add/3","title":"Luminous.TimeRange.add/3","type":"function"},{"doc":"","ref":"Luminous.TimeRange.html#default/1","title":"Luminous.TimeRange.default/1","type":"function"},{"doc":"Implement inside a client-side dashboard in order to return the dashboard's default time range.","ref":"Luminous.TimeRange.html#c:default_time_range/1","title":"Luminous.TimeRange.default_time_range/1","type":"callback"},{"doc":"","ref":"Luminous.TimeRange.html#default_time_zone/0","title":"Luminous.TimeRange.default_time_zone/0","type":"function"},{"doc":"","ref":"Luminous.TimeRange.html#from_iso/2","title":"Luminous.TimeRange.from_iso/2","type":"function"},{"doc":"","ref":"Luminous.TimeRange.html#from_unix/2","title":"Luminous.TimeRange.from_unix/2","type":"function"},{"doc":"","ref":"Luminous.TimeRange.html#last_month/2","title":"Luminous.TimeRange.last_month/2","type":"function"},{"doc":"","ref":"Luminous.TimeRange.html#last_n_days/3","title":"Luminous.TimeRange.last_n_days/3","type":"function"},{"doc":"","ref":"Luminous.TimeRange.html#last_week/2","title":"Luminous.TimeRange.last_week/2","type":"function"},{"doc":"","ref":"Luminous.TimeRange.html#new/2","title":"Luminous.TimeRange.new/2","type":"function"},{"doc":"","ref":"Luminous.TimeRange.html#round/2","title":"Luminous.TimeRange.round/2","type":"function"},{"doc":"","ref":"Luminous.TimeRange.html#shift_zone!/2","title":"Luminous.TimeRange.shift_zone!/2","type":"function"},{"doc":"","ref":"Luminous.TimeRange.html#this_month/2","title":"Luminous.TimeRange.this_month/2","type":"function"},{"doc":"","ref":"Luminous.TimeRange.html#this_week/2","title":"Luminous.TimeRange.this_week/2","type":"function"},{"doc":"","ref":"Luminous.TimeRange.html#to_map/1","title":"Luminous.TimeRange.to_map/1","type":"function"},{"doc":"","ref":"Luminous.TimeRange.html#today/2","title":"Luminous.TimeRange.today/2","type":"function"},{"doc":"","ref":"Luminous.TimeRange.html#tomorrow/2","title":"Luminous.TimeRange.tomorrow/2","type":"function"},{"doc":"","ref":"Luminous.TimeRange.html#yesterday/2","title":"Luminous.TimeRange.yesterday/2","type":"function"},{"doc":"","ref":"Luminous.TimeRange.html#t:t/0","title":"Luminous.TimeRange.t/0","type":"type"},{"doc":"","ref":"Luminous.TimeRange.html#t:time_zone/0","title":"Luminous.TimeRange.time_zone/0","type":"type"},{"doc":"A selector supports the widget in the dashboard that allows for selecting a time range/period. It can also be updated with a new value.","ref":"Luminous.TimeRangeSelector.html","title":"Luminous.TimeRangeSelector","type":"module"},{"doc":"Calculates and returns the time range for the given preset in the given time zone.","ref":"Luminous.TimeRangeSelector.html#get_time_range_for/2","title":"Luminous.TimeRangeSelector.get_time_range_for/2","type":"function"},{"doc":"","ref":"Luminous.TimeRangeSelector.html#hook/0","title":"Luminous.TimeRangeSelector.hook/0","type":"function"},{"doc":"","ref":"Luminous.TimeRangeSelector.html#id/0","title":"Luminous.TimeRangeSelector.id/0","type":"function"},{"doc":"Create and return a new selector","ref":"Luminous.TimeRangeSelector.html#new/1","title":"Luminous.TimeRangeSelector.new/1","type":"function"},{"doc":"Returns a list with the available time range presets.","ref":"Luminous.TimeRangeSelector.html#presets/0","title":"Luminous.TimeRangeSelector.presets/0","type":"function"},{"doc":"Updates the current time range of the selector.","ref":"Luminous.TimeRangeSelector.html#update_current/2","title":"Luminous.TimeRangeSelector.update_current/2","type":"function"},{"doc":"","ref":"Luminous.TimeRangeSelector.html#t:preset/0","title":"Luminous.TimeRangeSelector.preset/0","type":"type"},{"doc":"","ref":"Luminous.TimeRangeSelector.html#t:t/0","title":"Luminous.TimeRangeSelector.t/0","type":"type"},{"doc":"Various utility functions.","ref":"Luminous.Utils.html","title":"Luminous.Utils","type":"module"},{"doc":"","ref":"Luminous.Utils.html#dom_id/1","title":"Luminous.Utils.dom_id/1","type":"function"},{"doc":"","ref":"Luminous.Utils.html#print_number/1","title":"Luminous.Utils.print_number/1","type":"function"},{"doc":"A variable is defined inside a Dashboard and its values are determined at runtime. The variable also stores a current value that can be updated. A variable value can be simple (just a value) or descriptive in that it contains a label (for display purposes) and the actual value. Variables are visualized as dropdowns in the dashboard view. There are two Variable types: single : only one value can be selected by the user (default type) multi : multiple values can be selected by the user","ref":"Luminous.Variable.html","title":"Luminous.Variable","type":"behaviour"},{"doc":"Defines a new variable and returns a map. The following options can be passed: :id ( atom/0 ) - Required. :label ( String.t/0 ) - Required. :module ( atom/0 ) - Required. :type - The default value is :single .","ref":"Luminous.Variable.html#define!/1","title":"Luminous.Variable.define!/1","type":"function"},{"doc":"Extract and return the value from the descriptive variable value.","ref":"Luminous.Variable.html#extract_value/1","title":"Luminous.Variable.extract_value/1","type":"function"},{"doc":"Find and return the variable with the specified id in the supplied variables.","ref":"Luminous.Variable.html#find/2","title":"Luminous.Variable.find/2","type":"function"},{"doc":"Returns the variable's current (descriptive) value(s) or nil .","ref":"Luminous.Variable.html#get_current/1","title":"Luminous.Variable.get_current/1","type":"function"},{"doc":"Find the variable with the supplied id in the supplied variables and return its current extracted value.","ref":"Luminous.Variable.html#get_current_and_extract_value/2","title":"Luminous.Variable.get_current_and_extract_value/2","type":"function"},{"doc":"Returns the label based on the variable type and current value selection","ref":"Luminous.Variable.html#get_current_label/1","title":"Luminous.Variable.get_current_label/1","type":"function"},{"doc":"Uses the callback to populate the variables's values and returns the updated variable. Additionally, it sets the current value to be the first of the available values in the case of a single variable or all of the available values in the case of a multi variable.","ref":"Luminous.Variable.html#populate/2","title":"Luminous.Variable.populate/2","type":"function"},{"doc":"Replaces the variables current value with the new value and returns the map. It performs a check whether the supplied value is a valid value (i.e. exists in values). If it's not, then it returns the map unchanged. The special &quot;none&quot; case is for when the variable's type is :multi and none of the values are selected (empty list)","ref":"Luminous.Variable.html#update_current/2","title":"Luminous.Variable.update_current/2","type":"function"},{"doc":"A module must implement this behaviour to be passed as an argument to define!/1 . The function receives the variable id and the LV socket assigns.","ref":"Luminous.Variable.html#c:variable/2","title":"Luminous.Variable.variable/2","type":"callback"},{"doc":"","ref":"Luminous.Variable.html#t:descriptive_value/0","title":"Luminous.Variable.descriptive_value/0","type":"type"},{"doc":"","ref":"Luminous.Variable.html#t:simple_value/0","title":"Luminous.Variable.simple_value/0","type":"type"},{"doc":"","ref":"Luminous.Variable.html#t:t/0","title":"Luminous.Variable.t/0","type":"type"},{"doc":"Luminous Luminous is a framework for creating dashboards within Phoenix Live View . Dashboards are defined by the client application (framework consumer) using elixir code and consist of Panels ( Luminous.Panel ) which are responsible for visualizing the results of multiple client-side queries ( Luminous.Query ). Three different types of Panels are currently offered out of the box by Luminous: Luminous.Panel.Chart for visualizing 2-d data (including time series) using the chartjs library (embedded in a JS hook). Currently, only :line and :bar are supported. Luminous.Panel.Stat for displaying single or multiple numerical or other values (e.g. strings) Luminous.Panel.Table for displaying tabular data A client application can implement its own custom panels by implementing the Luminous.Panel behaviour. Dashboards are parameterized by: a date range (using the flatpickr library) user-defined variables ( Luminous.Variable ) in the form of dropdown menus All panels are refreshed whenever at least one of these paramaters (date range, variables) change. The parameter values are available to client-side queries.","ref":"readme.html","title":"README","type":"extras"},{"doc":"Date range selection and automatic asynchronous (i.e. non-blocking for the UI) refresh of all dashboard panel queries User-facing variable dropdowns (with single- or multi- selection) whose selected values are available to panel queries Client-side zoom in charts with automatic update of the entire dashboard with the new date range Panel data downloads depending on the panel type (CSV, PNG) Stat panels (show single or multiple stats) Table panels using tabulator Summary statistics in charts","ref":"readme.html#features","title":"README - Features","type":"extras"},{"doc":"The package can be installed from hex.pm as follows: def deps do [ { :luminous , &quot;~&gt; 2.4.0&quot; } ] end In order to be able to use the provided components, the library's javascript and CSS files must be imported to your project: In assets/js/app.js : import { ChartJSHook, TableHook, TimeRangeHook, MultiSelectVariableHook } from &quot;luminous&quot; let Hooks = { TimeRangeHook: new TimeRangeHook(), ChartJSHook: new ChartJSHook(), TableHook: new TableHook(), MultiSelectVariableHook: new MultiSelectVariableHook() } ... let liveSocket = new LiveSocket(&quot;/live&quot;, Socket, { ... hooks: Hooks }) ... Finally, in assets/css/app.css : @import &quot;../../deps/luminous/dist/luminous.css&quot;;","ref":"readme.html#installation","title":"README - Installation","type":"extras"},{"doc":"Live View The dashboard live view is defined client-side like so: defmodule ClientApp.DashboardLive do alias ClientApp.Router.Helpers , as : Routes use Luminous.Live , title : &quot;My Title&quot; , path : &amp; Routes . dashboard_path / 3 , action : :index , time_zone : &quot;Europe/Paris&quot; , panels : [ ... ] , variables : [ ... ] # the dashboard can be rendered by leveraging the corresponding functionality # from `Luminous.Components` def render ( assigns ) do ~H&quot;&quot;&quot; &lt;Luminous.Components.dashboard dashboard={@dashboard} /&gt; &quot;&quot;&quot; end end The client-side dashboard can also (optionally) implement the Luminous.TimeRange behaviour in order to override the dashboard's default time range value which is &quot;today&quot;. Panels and Queries Client-side queries must be included in a module that implements the Luminous.Query behaviour: defmodule ClientApp.DashboardLive do defmodule Queries do @behaviour Luminous.Query @impl true def query ( :my_query , _time_range , _variables ) do [ [ { :time , ~U[2022-08-19T10:00:00Z] } , { &quot;foo&quot; , 10 } , { &quot;bar&quot; , 100 } ] , [ { :time , ~U[2022-08-19T11:00:00Z] } , { &quot;foo&quot; , 11 } , { &quot;bar&quot; , 101 } ] ] end end use Luminous.Live , ... panels : [ Panel . define! ( type : Luminous.Panel.Chart , id : :simple_time_series , title : &quot;Simple Time Series&quot; , queries : [ Luminous.Query . define ( :my_query , Queries ) ] , description : &quot;&quot;&quot; This will be rendered as a tooltip when hovering over the panel&#39;s title &quot;&quot;&quot; ) , ] , ... end A panel may include multiple queries. When a panel is automatically refreshed, the execution flow is as follows: for each query: execute the user query callback execute the panel's transform/2 callback with the query result output aggregate the transformed query results update the dashboard state variable with the panel's data (possible server-side re-rendering) send a JS event to the browser (for panel hooks) The above flow needs to be understood when implementing custom panels. If the client application uses the panels provided by luminous, then the panel refresh flow is handled automatically and only use Luminous.Live with the appropriate options is necessary. Variables Variables represent user-facing elements in the form of dropdowns in which the user can select single (variable type: :single ) or multiple (variable type: :multi ) values. Variable selections trigger the refresh of all panels in the dashboard. The state of all variables is available within the query callback that is implemented by the client application. Just like queries, variables must be included in a module that implements the Luminous.Variable behaviour: defmodule ClientApp.DashboardLive do defmodule Variables do @behaviour Luminous.Variable @impl true def variable ( :simple_var , _assigns ) , do : [ &quot;hour&quot; , &quot;day&quot; , &quot;week&quot; ] def variable ( :descriptive_var , _assigns ) do [ %{ label : &quot;Visible Value 1&quot; , value : &quot;val1&quot; } , %{ label : &quot;Visible Value 2&quot; , value : &quot;val2&quot; } , ] end end use Luminous.Live , ... variables : [ Luminous.Variable . define! ( id : :simple_var , label : &quot;Select one value&quot; , module : Variables ) , Luminous.Variable . define! ( id : :descriptive_var , label : &quot;Select one value&quot; , module : Variables ) , ] , ... end The variable callback will receive the live view socket assigns as the second argument, however it is important to note that the variable/2 callback is executed once when the dashboard is loaded for populating the dropdown values. Demo Luminous provides a demo dashboard that showcases some of Luminous' capabilities. The demo dashboard can be inspected live using the project's development server (run mix run in the project and then visit this page ).","ref":"readme.html#usage","title":"README - Usage","type":"extras"},{"doc":"luminous provides CSS classes that can be overriden, so that the components match the look and feel of the consumer application. Those classes belong to three luminous components: Variables Time range selector Panel dropdown Variables lmn-variable-button Define the size, shape, background color, on hover behavior, etc. of the variable buttons. lmn-variable-button-label Define the font, text size, weight, alignment of the variable button's label. lmn-variable-button-label-prefix Define the font, text size, weight, alignment of the variable button's label prefix. lmn-variable-button-icon Define the size and alignment of the variable button's chevron icon. lmn-variable-dropdown Define the size, background color, rouding, shadows, etc. of the variable dropdown menu. lmn-variable-dropdown-item-container Define the text alignment, rounding, on hover behaviour, etc. of each item in the variable dropdown menu. lmn-variable-dropdown-item-content Define the size, padding, etc. of the content of each item in the variable dropdown menu. Time range picker lmn-time-range-compound Define the structure of the time range component. This includes the time range picker button, the pressets button and the time zone component. lmn-time-range-selector Define the structure, the size and the shape of the time range selector component. This includes the button that opens the custom time range selector and the button that opens the preset menu dropdown. lmn-custom-time-range-input Define the size, background color, text size, on hover behavior, etc. of the button that opens the custom date range picker dropdown. lmn-time-range-presets-button Define the size, background color, on hover behavior, etc. of the button that opens the time range presets dropdown. lmn-time-range-presets-button-icon Define the size and spacing of the icon in the button that opens the time range presets dropdown. lmn-time-range-presets-dropdown Define the size, background color, rouding, shadows, etc. of the time range presets dropdown menu. lmn-time-range-presets-dropdown-item-container Define the text alignment, rounding, on hover behaviour, etc. of each item in the time range presets dropdown menu. lmn-time-range-presets-dropdown-item-content Define the size, padding, etc. of the content of each item in the time range presets dropdown menu. lmn-time-zone Define the background color, rounding, text size, etc. of the time zone label Panel dropdown lmn-panel-actions-dropdown Define the size, background color, rouding, shadows, etc. of the panel actions dropdown menu. lmn-panel-actions-dropdown-item-container Define the text alignment, rounding, on hover behaviour, etc. of each item in the panel actions dropdown menu. lmn-panel-actions-dropdown-item-content Define the size, padding, etc. of the content of each item in the panel actions dropdown menu. Dropdown transition lmn-dropdown-transition-enter Define the animation of all dropdowns when they open up. lmn-dropdown-transition-start Define the initial state of the animation of all dropdowns when they open up. lmn-dropdown-transition-end Define the final state of the animation of all dropdowns when they open up. Calendar dropdown For the calendar dropdown, by default we use the airbnb theme provided by flatpickr . To change this theme, you have to import the desired theme after importing luminous to your app.css file like so: @import 'luminous/dist/luminous'; @import '../node_modules/flatpickr/dist/themes/material_blue.css'; The path that the flatpickr theme resides, depends on your project's directory structure.","ref":"applying-custom-css.html","title":"Applying custom CSS","type":"extras"}]